#!/usr/bin/python
# -*- coding:utf-8 -*-
#
#   Copyright 2008-2009 Le Coz Florent <louizatakk@fedoraproject.org>
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the WTF Public License as published by
#   Sam Hocevar.
#
#   DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
#   Version 2, December 2004
#   DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
#   TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION
#   0. You just DO WHAT THE FUCK YOU WANT TO.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#

import sys, os, xmpp, time, select
import pygame as pg
from ownevents import *
import threading

if len(sys.argv) > 1 and sys.argv[1] == '--debug':
    DEBUG = True
else:
    DEBUG = False

class Jabber(threading.Thread):
    '''Handle everything related to network'''
    def __init__(self, jid, passwd, tojid, config, deck):
        threading.Thread.__init__(self)
        self.debug = 0
        self.debug2 = 0
        self.config = config
        self.deck = deck
        self.jid = jid+'/ludimagia'
        self.jid = xmpp.protocol.JID(self.jid)
        if tojid.strip() == '':
            self.play_connected = False
            self.connected = True
            self.tojid = ''
            return
        else:
            self.play_connected = True
        self.tojid = tojid+'/ludimagia'

        self.jid = xmpp.protocol.JID(self.jid)

        self.passwd = passwd
        self.remotejid = tojid
        self.connected = False
        
        self.client = xmpp.Client(self.jid.getDomain(), debug=[])

    def run(self):
        '''Really run the instance in a thread'''
        if not self.xmpp_connect():
            sys.stderr.write("Could not connect to server, or password mismatch!\n")
            sys.exit(1)
        online = True

        while online:
            self.client.Process(3)
        self.jabber.disconnect()
        return

    def xmpp_connect(self):
        if self.play_connected == False:
            return
        try :
            server = self.config['server']
            if server == '':
                server = self.jid.getDomain()
            port = self.config['port']
            if port == '':
                port = 5222
        except:
            server = self.jid.getDomain()
            port = 5222
        try:
            prox_passw = self.config['prox_passwd']
            prox_host = self.config['prox_host']
            prox_user = self.config['prox_user']
            prox_port = self.config['prox_port']
            if prox_user != '' and prox_host != '' and prox_passwd != '':
                con = self.client.connect((server, port), {'hosts':prox_host, 'port':prox_port,
                                                           'user':prox_user, 'password':prox_passwd})
            else:
                con = self.client.connect((server, port))
        except:
            con = self.client.connect((server, port))
        if not con:
            sys.stderr.write('could not connect!\n')
            return False
        sys.stderr.write('connected with %s\n'%con)
        auth = self.client.auth(self.jid.getNode(), self.passwd, resource=self.jid.getResource())
        if not auth:
            sys.stderr.write('could not authenticate!\n')
            return False
        sys.stderr.write('Success! Authenticated using %s\n' % auth)
        self.register_handlers()
        self.send_init(len(self.deck))
        return con

    def register_handlers(self):
        self.client.RegisterHandler('message', self.xmpp_message)

    def xmpp_message(self, con, event):
        type = event.getType()
        if DEBUG:
            print 'Receiving #%s' % (self.debug2), event
        self.debug2 += 1
        fromjid = event.getFrom().getStripped()
        if not self.connected:
            if type == 'init':
                if self.connected == False:
                    deck_size = event.T.move.T.id.getCDATA()
                    pg.event.post(pg.event.Event(EVENTINIT, {'deck_size': deck_size}))
                    self.send_init(len(self.deck))
        else:
            if type == 'message':
                msg = event.T.msg.getCDATA()
                pg.event.post(pg.event.Event(EVENTMSG, {'message':msg}))
            elif type == 'move':
                id = event.T.move.T.id.getDATA()
                x = event.T.move.T.x.getDATA()
                y = event.T.move.T.y.getDATA()
                pg.event.post(pg.event.Event(EVENTMOVE, {'id':id, 'x':x, 'y':y}))
            elif type == 'draw':
                id = event.T.id.getCDATA()
                pg.event.post(pg.event.Event(EVENTDRAW, {'id':id}))
            elif type == 'undraw':
                id = event.T.id.getCDATA()
                pg.event.post(pg.event.Event(EVENTUNDRAW, {'id':id}))
            elif type == 'edit':
                id = event.T.id.getCDATA()
                x = event.T.x.getCDATA()
                y = event.T.y.getCDATA()
                where = event.T.where.getCDATA()
                name = event.T.name.getCDATA()
                tapped = event.T.tapped.getCDATA()
                pg.event.post(pg.event.Event(EVENTEDIT, {'id':id, 'x':x, 'y':y, 'where':where,
                                                         'name':name, 'tapped':tapped}))
            elif type == 'life':
                nb = event.T.nb.getCDATA()
                pg.event.post(pg.event.Event(EVENTLIFE, {'nb':nb}))
            elif type == 'untap_all':
                pg.event.post(pg.event.Event(EVENTUNTAP_ALL, {}))

    def send(self, ms):
        if DEBUG:
            print 'Sending #%s' % (self.debug), ms
        self.debug += 1
        if self.play_connected == True:
            self.client.send(ms)
        else:
            return

    def send_init(self, size):
        ms = xmpp.protocol.Message(to=self.tojid, typ='init')
        ms.NT.move = None
        ms.T.move.T.id = str(size)
        self.send(ms)

    def send_move(self, id, pos):
        '''pos: tuple'''
        # not used anymore: remove me
        ms = xmpp.protocol.Message(to=self.tojid, typ='move')
        ms.NT.move = None
        ms.T.move.T.id = id
        ms.T.move.T.x = pos[0]
        ms.T.move.T.y = pos[1]
        self.send(ms)

    def send_draw(self, id):
        ms = xmpp.protocol.Message(to=self.tojid, typ='draw')
        ms.NT.id = str(id)
        self.send(ms)

    def send_undraw(self, id):
        ms = xmpp.protocol.Message(to=self.tojid, typ='undraw')
        ms.NT.id = str(id)
        self.send(ms)

    def send_message(self, message):
        ms = xmpp.protocol.Message(to=self.tojid, typ='message')
        ms.NT.msg = message
        self.send(ms)

    def send_life_change(self, nb):
        ms = xmpp.protocol.Message(to=self.tojid, typ='life')
        ms.NT.nb = nb
        self.send(ms)

    def send_untap_all(self):
        ms = xmpp.protocol.Message(to=self.tojid, typ='untap_all')
        self.send(ms)

    def send_edit_card(self, dic):
        ms = xmpp.protocol.Message(to=self.tojid, typ='edit')
        try:
            ms.NT.id = dic['id']
        except:
            ms.NT.id = None
        try:
            ms.NT.x = dic['x']
        except: ms.NT.x = None
        try:
            ms.NT.y = dic['y']
        except: ms.NT.y = None
        try:
            ms.NT.where = dic['where']
        except: ms.NT.where = None
        try:
            ms.NT.tapped = dic['tapped']
        except: ms.NT.tapped = None
        try:
            ms.NT.name = dic['name']
        except: ms.NT.name = None
        self.send(ms)
